#include <linux/module.h>
#include <linux/device.h>
#include <linux/platform_device.h>
#include <linux/gpio_event.h>

int find_ms2_dev(struct device *dev, void *data)
{
	if (!strncmp((char*)data, dev_name(dev), strlen((char*)data))) {
		printk(KERN_INFO "Found it!\n");
		return 1;
	}
	return 0;
}

static int __init debounce_init(void)
{
	struct device *event_dev = NULL;
	struct gpio_event_platform_data *gpio_epd;
	struct gpio_event_info *gpio_ei;
	struct gpio_event_matrix_info *gpio_evmi;

	printk(KERN_INFO "Searching for " GPIO_EVENT_DEV_NAME "...\n");

	
	event_dev = device_find_child(&platform_bus, GPIO_EVENT_DEV_NAME, find_ms2_dev);
	if (event_dev == NULL)
		return -ENODEV;

	gpio_epd = (struct gpio_event_platform_data*)event_dev->platform_data;
	printk(KERN_INFO "And there is a %s connected...\n", gpio_epd->name);
	if (strcmp(gpio_epd->name, "sholes-keypad"))
		return -ENODEV;

	gpio_ei = (struct gpio_event_info*)gpio_epd->info[0];
	gpio_evmi = container_of(gpio_ei, struct gpio_event_matrix_info, info);

	printk(KERN_INFO "settle_time: %u\n", gpio_evmi->settle_time.tv.nsec);
	printk(KERN_INFO "poll_time: %u\n", gpio_evmi->poll_time.tv.nsec);
	printk(KERN_INFO "debounce_delay: %u\n", gpio_evmi->debounce_delay.tv.nsec);
	printk(KERN_INFO "flags: %x\n", gpio_evmi->flags);
	if (!gpio_evmi->debounce_delay.tv.nsec) {
		printk(KERN_INFO "Activating debounce!\n");
		gpio_evmi->debounce_delay.tv.nsec = 5 * NSEC_PER_MSEC;

		/* GPIOKPF_DEBOUNCE should already be set by GPIOKPF_REMOVE_PHANTOM_KEYS */
		gpio_evmi->flags |= GPIOKPF_DEBOUNCE;
		printk(KERN_INFO "debounce_delay: %u\n", gpio_evmi->debounce_delay.tv.nsec);
		printk(KERN_INFO "flags: %x\n", gpio_evmi->flags);
	}

	return 0;
}

static void __exit debounce_exit(void)
{
}

module_init(debounce_init);
module_exit(debounce_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Michael Gernoth <michael@gernoth.net>");
